//
// This file was modified from the original implementation of altera_avalon_remote_update_cycloneiii.v
//

/*****************************************************************************/
// altera_avalon_remote_update_cyclone5.v
//
// This verilog file is a lightweight wrapper for the Cyclone V remote_update
// block.  Using this wrapper, the remote update block can be imported into 
// SOPC Builder using Component Editor.
//  
// Modifications made with this wrapper:
//
// 1.) The data_in and data_out ports of the remote update block have been 
//     expanded to 32-bits to comply with SOPC Builder 
// 2.) A register has been added so that the reconfig and reset_timer control 
//     bits of the remote update block can be controlled from an Avalon master
//     (such as NiosII)
// 3.) Removed resd source - not support in the V families
// 4.) Added busylastclock to drag out waitrequest for an extra clock cycle
//     in order to properly capture data_out from altremote_update_cyclone5.
// 5.) Changed address [2:0] description to match Cyclone V rather than 
//     Cyclone III,IV
/*****************************************************************************/


module rsu_cyclone5 (
  clk,
  reset,
  writedata,
  readdata,
  address,
  chipselect,
  write,
  read,
  waitrequest );
  
  
  input clk;
  input reset;
  input [31:0] writedata;
  output [31:0] readdata;
  input [5:0] address;
  input chipselect;
  input write;
  input read;
  output waitrequest;
  

  /************************************************************************
  **  Address[2:0] description
  *************************************************************************

    000 - Reconfiguration trigger conditions (Read Only) - Width of 5
    001 - (Illegal Value)
    010 - Watchdog Timeout Value - width of 12 when reading 
    011 - Watchdog Enable - width of 1
    100 - Page Select - width of 3 when reading and when writing the page 
                        (for Arria GX, Stratix, Stratix GX, Stratix II, and Stratix II GX Only)
                      - width of 7 when reading and writing the start address
                      - width of 24 when reading and writing the start address
    101 - Configuration Mode (AnF) - width of 1
    110 - (Illegal Value)
    111 - (Illegal Value)

  *************************************************************************
  **  Address[4:3] description
  *************************************************************************
   

   NOT USED FOR CYCLONE V ARRIA V AND STRATIX V   
    
  
  *************************************************************************
  **  Address[5] description
  *************************************************************************

		Enables access to control_status_reg
		  -Used to control 'reconfig' and 'reset_timer' signals from Avalon
		  
  ************************************************************************/
`ifdef QUARTUS__SIMGEN
  assign waitrequest = 1'b0;
  assign readdata = 32'b0;
`else

  wire [2:0] param_wire = address[2:0];
//  wire [1:0] read_source = address[4:3];
  wire read_param;
  wire write_param;
  //wire [21:0] data_in = writedata[21:0];
  wire [23:0] data_out;
  wire reconfig;
  wire reset_timer;
  wire reset_n = ~reset;
  wire waitrequest;
  wire busy;

  reg readorwritelastclock;
  reg busylastclock;
  reg [31:0] readdata;
  reg [31:0] control_status_reg;

  assign read_param = chipselect & read & ~address[5] & ~readorwritelastclock;
  assign write_param = chipselect & write & ~address[5] & ~readorwritelastclock;
  assign waitrequest = (( busy ^ (( write | read ) & ~readorwritelastclock )) | busylastclock );

  /*****************************************************************************/
  // General Purpose Register - Read/Writeable by Avalon master
  // Currently used to provide bus access to Reconfig / Watchdog Reset bits
  //  Offset = 0x10000
  /*****************************************************************************/
  always @(posedge clk or negedge reset_n)
  if (~reset_n)
    control_status_reg <= 0;
  else if ( chipselect & write & ( address == 6'b100000 ))
    control_status_reg <= writedata;
  else
    control_status_reg <= control_status_reg;


  // Register read logic
  always @(posedge clk or negedge reset_n)
    if( ~reset_n )
      readdata <= 0;
    else if( chipselect & read & ~address[5] )
      readdata <= {8'b00000000, data_out};
    else if( chipselect & read & ( address == 6'b100000 ))
      readdata <= control_status_reg;
    else
      readdata <= readdata;
      
//  assign readdata = address[5] ? control_status_reg: {4'b0000, data_out} ;

  // Delayed 'OR' of the read and write signals
  // Used to pulse the read_param/write_param signals for just one clock cycle
  always @(posedge clk or negedge reset_n)
  if (~reset_n)
    readorwritelastclock <= 0;
  else 
    readorwritelastclock <= ( read | write );

  // Delayed busy signal
  // Used to hold waitrequest for one more cycle
  always @(posedge clk or negedge reset_n)
  if (~reset_n)
    busylastclock <= 0;
  else 
    busylastclock <= busy;

  
  assign reconfig = control_status_reg[0];
  assign reset_timer = control_status_reg[1];

  /*****************************************************************************/
  // Instantiation of remote update funciton.
  /*****************************************************************************/        
altremote_update_cyclone5	altremote_update_cyclone5_inst (
	.clock ( clk ),
	.data_in ( writedata[23:0] ),
	.param ( param_wire ),
	.read_param ( read_param ),
//	.read_source ( read_source ),
	.reconfig ( reconfig ),
	.reset ( reset ),
	.reset_timer ( reset_timer ),
	.write_param ( write_param ),
	.busy ( busy ),
	.data_out ( data_out )
	);

`endif

endmodule
